-----------------------
--- Module Creation ---
-----------------------
local core = LibStub("AceAddon-3.0"):GetAddon("tomQuest2")
local module = core:NewModule("questsGivers", "AceEvent-3.0")

-----------------------
--- Library loading ---
-----------------------
local LQG = LibStub("AceLocale-3.0"):GetLocale("tomQuest2-QuestsGivers")
local LDQ = LibStub("AceLocale-3.0"):GetLocale("tomQuest2-DailyQuests")
local LFQ = LibStub("AceLocale-3.0"):GetLocale("tomQuest2-ForbidenQuests")

--------------------------------------------
--- Copy global functions to local scope ---
--------------------------------------------
local select, pairs, format = select, pairs, string.format
local GetGossipAvailableQuests, GetGossipActiveQuests = GetGossipAvailableQuests, GetGossipActiveQuests
local GetNumActiveQuests, GetNumAvailableQuests = GetNumActiveQuests, GetNumAvailableQuests
local GetActiveTitle, GetActiveLevel = GetActiveTitle, GetActiveLevel
local GetAvailableTitle, GetAvailableLevel = GetAvailableTitle, GetAvailableLevel
local AcceptQuest, IsShiftKeyDown, GetTitleText, GetNumQuestChoices, GetQuestReward, IsQuestCompletable, CompleteQuest, ConfirmAcceptQuest = AcceptQuest, IsShiftKeyDown, GetTitleText, GetNumQuestChoices, GetQuestReward, IsQuestCompletable, CompleteQuest, ConfirmAcceptQuest

-----------------
--- variables ---
-----------------
local prdb, options
local trivialText = "|cff%02x%02x%02x[%s]|r "..TRIVIAL_QUEST_DISPLAY
local normalText = "|cff%02x%02x%02x[%s]|r "..NORMAL_QUEST_DISPLAY

module.modName = LQG["Quests givers"]

module.dailies = {}
module.automaticAcceptChoices = {[1] = LQG["All quests"], [2] = LQG["Only daily quests"], [3] = LQG["All but daily quests"], [4] = LQG["Never"]}

-----------------------
--- Default profile ---
-----------------------
local defaults = {
	profile = {
		addQuestLevel = true,
		fadeNotCompletedQuests = true,
		automaticAccept = 4,
		automaticAcceptEscort = false,
		automaticTurnIn = false,
	}
}

---------------
--- Options ---
---------------
local optGetter, optSetter
do
	function optGetter(info)
		return prdb[info[#info]]
	end
	function optSetter(info, value)
		prdb[info[#info]] = value
	end
end

local function getOptions()
	if not options then
		options = {
			type = "group",
			childGroups = "tab",
			name = module.modName,
			get = optGetter,
			set = optSetter,
			args = {
				addQuestLevel = {
					name = LQG["Add quest level to the quest name"],
					type = "toggle",
					width = "full",
					order = 1,
				},
				fadeNotCompletedQuests = {
					name = LQG["Fade quest name that are not complete"],
					type = "toggle",
					width = "full",
					order = 2,
				},
				automaticAccept = {
					name = LQG["Accept quests automatically"],
					type = "select",
					values = module.automaticAcceptChoices,
					width = "full",
					order = 3,
				},
				automaticAcceptEscort = {
					name = LQG["Accept escort quest started by other player automatically"],
					type = "toggle",
					width = "full",
					order = 4,
				},
				automaticTurnIn = {
					name = LQG["Automatic quest turn-in"],
					type = "toggle",
					width = "full",
					order = 5,
				},
			},
		}
	end
	return options
end

function module:OnInitialize()
	self.db = core.db:RegisterNamespace("questsGivers", defaults, "Default")
	prdb = self.db.profile
	self:SetEnabledState(core:getModuleEnabled("questsGivers"))
	core.moduleState["questsGivers"] = core:getModuleEnabled("questsGivers")
	core:registerModuleOptions("questsGivers", getOptions, module.modName)
end

function module:OnEnable()
	for uid, v in pairs(LDQ) do
		self.dailies[v] = true
	end
	self:RegisterEvent("GOSSIP_SHOW")
	self:RegisterEvent("QUEST_GREETING")
	self:RegisterEvent("QUEST_COMPLETE")
	self:RegisterEvent("QUEST_DETAIL")
	self:RegisterEvent("QUEST_PROGRESS")
	self:RegisterEvent("QUEST_ACCEPT_CONFIRM")
end

function module:clickQuest(button)
	if button:IsVisible() then
		local title = button:GetText()
		title = title:gsub("|c%x%x%x%x%x%x%x%x(.+)|r","%1")
		title = title:gsub("%[(.+)%] ", "")
		title = title:gsub("(.+) %((.+)%)", "%1")
		if button.type == "Available" and (prdb.automaticAccept == 1 or (prdb.automaticAccept == 2 and self.dailies[title]) or (prdb.automaticAccept == 3 and not self.dailies[title])) then
			button:Click()
		elseif button.type == "Active" and prdb.automaticTurnIn and ((core.questsTitleToUid[title] and (core.quests[core.questsTitleToUid[title]].complete[core.playerName] == 1 or core.quests[core.questsTitleToUid[title]].numObjectives == 0)) or (not core.questsTitleToUid[title])) then
			button:Click()
		end
	end
end

function module:loopAvailableQuests(...)
	local numQuests = select('#', ...)
	if numQuests == 0 then return end
	for i = 1, numQuests, 3 do
		self.buttonIndex = self.buttonIndex + 1
		if prdb.addQuestLevel then
			local title, level, trivial = select(i, ...)
			local color = core:getDifficultyColor(level - core.playerLevel)
			core:Print("r"..color.r)
			core:Print("g"..color.g)
			core:Print("b"..color.b)
			local buttonName = "GossipTitleButton"..self.buttonIndex
			_G[buttonName]:SetFormattedText(trivial and trivialText or normalText, 255*color.r, 255*color.g, 255*color.b, level == -1 and "*" or level, title)
			_G[buttonName.."GossipIcon"]:SetVertexColor(1, 1, 1, 1)
		end
	end
end

function module:loopActiveQuests(...)
	local numQuests = select('#', ...)
	if numQuests == 0 then return end
	for i = 1, numQuests, 3 do
		self.buttonIndex = self.buttonIndex + 1
		local buttonName = "GossipTitleButton"..self.buttonIndex
		local title, level, trivial = select(i, ...)
		core:Print(self.buttonIndex)
		core:Print("title"..title)
		core:Print("level"..level)
		if prdb.addQuestLevel then
			local color = core:getDifficultyColor(level - core.playerLevel)
			core:Print("r"..color.r)
			core:Print("g"..color.g)
			core:Print("b"..color.b)
			_G[buttonName]:SetFormattedText(trivial and trivialText or normalText, 255*color.r, 255*color.g, 255*color.b, level == -1 and "*" or level, title)
		end
		if prdb.fadeNotCompletedQuests then
			local uid = core.questsTitleToUid[title]
			if uid and core.quests[uid].playerComplete == 0 then
				_G[buttonName.."GossipIcon"]:SetVertexColor(0.5, 0.5, 0.5, 0.5)
			else
				_G[buttonName.."GossipIcon"]:SetVertexColor(1, 1, 1, 1)
			end
		else
			_G[buttonName.."GossipIcon"]:SetVertexColor(1, 1, 1, 1)
		end
	end
	self.buttonIndex = self.buttonIndex + 1
end

function module:GOSSIP_SHOW()
	self.buttonIndex = 0
	self:loopAvailableQuests(GetGossipAvailableQuests())
	self:loopActiveQuests(GetGossipActiveQuests())
	if IsShiftKeyDown() then return end
	for i = 1, self.buttonIndex do
		self:clickQuest(_G["GossipTitleButton" .. i])
	end
end

function module:QUEST_GREETING()
	local numActive, numAvailable = GetNumActiveQuests() or 0, GetNumAvailableQuests() or 0
	core:Print("Greeting")
	for i = 1, numActive do
		local buttonName = "QuestTitleButton"..i
		local title = GetActiveTitle(i)
		if prdb.addQuestLevel then
			local level = GetActiveLevel(i)
			core:Print("r"..color.r)
			core:Print("g"..color.g)
			core:Print("b"..color.b)
			_G["QuestTitleButton"..(i + numActive)]:SetText("["..(level == -1 and "*" or level).."] "..GetAvailableTitle(i))
		end
		if prdb.fadeNotCompletedQuests then
			local uid = core.questsTitleToUid[title]
			if uid and core.quests[uid].playerComplete == 0 then
				_G[buttonName.."QuestIcon"]:SetVertexColor(0.5, 0.5, 0.5, 0.5)
			else
				_G[buttonName.."QuestIcon"]:SetVertexColor(1, 1, 1, 1)
			end
		else
			_G[buttonName.."QuestIcon"]:SetVertexColor(1, 1, 1, 1)
		end
	end
	if IsShiftKeyDown() then return end
	for i = 1, numActive + numAvailable do
		self:clickQuest(_G['QuestTitleButton' .. i])
	end
end

function module:QUEST_DETAIL()
	local title = GetTitleText()
	if not IsShiftKeyDown() and (prdb.automaticAccept == 1 or (prdb.automaticAccept == 2 and self.dailies[title]) or (prdb.automaticAccept == 3 and not self.dailies[title])) then
		AcceptQuest()
	end
end

function module:QUEST_COMPLETE(event)
	if not IsShiftKeyDown() and prdb.automaticTurnIn and GetNumQuestChoices() <= 1 then
		GetQuestReward(QuestFrameRewardPanel.itemChoice)
	end
end

function module:QUEST_PROGRESS()
	if not IsShiftKeyDown() and prdb.automaticTurnIn and IsQuestCompletable() then
		CompleteQuest()
	end
end

function module:QUEST_ACCEPT_CONFIRM()
	if not IsShiftKeyDown() and prdb.automaticAcceptEscort then
		ConfirmAcceptQuest()
	end
end